@@ -21,6 +21,10 @@ extern "C" {
 	void WinPortHelp();
 	const wchar_t *WinPortBackend();
 
+	// true means far2l runs under smoke testing and code must
+	// not skip events from input queue that sometimes used to make UX smoother
+	BOOL WinPortTesting();
+
 	///console API
 	WINPORT_DECL(ForkConsole,HANDLE,());
 	WINPORT_DECL(JoinConsole,VOID,(HANDLE hConsole));
@@ -48,6 +52,7 @@ extern "C" {
 	WINPORT_DECL(FillConsoleOutputAttribute, BOOL, (HANDLE hConsoleOutput, DWORD64 qAttributes, DWORD nLength, COORD dwWriteCoord, LPDWORD lpNumberOfAttrsWritten));
 	WINPORT_DECL(FillConsoleOutputCharacter, BOOL, (HANDLE hConsoleOutput, WCHAR cCharacter, DWORD nLength, COORD dwWriteCoord, LPDWORD lpNumberOfCharsWritten));
 	WINPORT_DECL(SetConsoleActiveScreenBuffer, BOOL,(HANDLE hConsoleOutput));
+	WINPORT_DECL(SetConsoleCursorBlinkTime,VOID,(HANDLE hConsoleOutput, DWORD dwMilliseconds ));
 
 	WINPORT_DECL(FlushConsoleInputBuffer,BOOL,(HANDLE hConsoleInput));
 	WINPORT_DECL(GetNumberOfConsoleInputEvents,BOOL,(HANDLE hConsoleInput, LPDWORD lpcNumberOfEvents));
@@ -95,6 +100,7 @@ extern "C" {
 #define TWEAK_STATUS_SUPPORT_OSC52CLIP_SET	0x04
 #define TWEAK_STATUS_SUPPORT_CHANGE_FONT	0x08
 #define TWEAK_STATUS_SUPPORT_TTY_PALETTE	0x10
+#define TWEAK_STATUS_SUPPORT_BLINK_RATE		0x20
 
 	WINPORT_DECL(SaveConsoleWindowState,VOID,());
 	WINPORT_DECL(ConsoleChangeFont, VOID, ());
