@@ -80,7 +80,7 @@ static WORD WChar2WinVKeyCode(WCHAR wc)
 }
 
 
-TTYBackend::TTYBackend(const char *full_exe_path, int std_in, int std_out, bool ext_clipboard, bool norgb, const char *nodetect, bool far2l_tty, unsigned int esc_expiration, int notify_pipe, int *result) :
+TTYBackend::TTYBackend(const char *full_exe_path, int std_in, int std_out, bool ext_clipboard, bool norgb, DWORD nodetect, bool far2l_tty, unsigned int esc_expiration, int notify_pipe, int *result) :
 	_full_exe_path(full_exe_path),
 	_stdin(std_in),
 	_stdout(std_out),
@@ -207,6 +207,16 @@ void TTYBackend::UpdateBackendIdentification()
 	g_winport_backend = s_backend_identification;
 }
 
+static bool UnderWayland()
+{
+	const char *xdg_st = getenv("XDG_SESSION_TYPE");
+	if (xdg_st && strcasecmp(xdg_st, "wayland") == 0)
+		return true;
+	if (getenv("WAYLAND_DISPLAY"))
+		return true;
+	return false;
+}
+
 void TTYBackend::ReaderThread()
 {
 	bool prev_far2l_tty = false;
@@ -221,8 +231,10 @@ void TTYBackend::ReaderThread()
 			}
 
 		} else {
-			if (!strchr(_nodetect, 'x') || strstr(_nodetect, "xi")) {
-				_ttyx = StartTTYX(_full_exe_path, !strstr(_nodetect, "xi"));
+			if ((_nodetect & NODETECT_X)==0) {
+
+				// disable xi on Wayland as it not work there anyway and also causes delays
+				_ttyx = StartTTYX(_full_exe_path, ((_nodetect & NODETECT_XI)==0) && !UnderWayland());
 			}
 			if (_ttyx) {
 				if (!_ext_clipboard) {
@@ -295,6 +307,11 @@ void TTYBackend::ReaderLoop()
 
 		int rs;
 
+		// Enable esc expiration on Wayland as Xi not work there
+		if (!_esc_expiration && UnderWayland()) {
+			_esc_expiration = 100;
+		}
+
 		if (!idle_expired && _esc_expiration > 0 && !_far2l_tty) {
 			struct timeval tv;
 			tv.tv_sec = _esc_expiration / 1000;
@@ -827,6 +844,11 @@ void TTYBackend::OnConsoleSaveWindowState()
 {
 }
 
+void TTYBackend::OnConsoleSetCursorBlinkTime(DWORD interval)
+{
+
+}
+
 void TTYBackend::OnConsoleSetMaximized(bool maximized)
 {
 	try {
@@ -1220,7 +1242,7 @@ static void OnSigHup(int signo)
 }
 
 
-bool WinPortMainTTY(const char *full_exe_path, int std_in, int std_out, bool ext_clipboard, bool norgb, const char *nodetect, bool far2l_tty, unsigned int esc_expiration, int notify_pipe, int argc, char **argv, int(*AppMain)(int argc, char **argv), int *result)
+bool WinPortMainTTY(const char *full_exe_path, int std_in, int std_out, bool ext_clipboard, bool norgb, DWORD nodetect, bool far2l_tty, unsigned int esc_expiration, int notify_pipe, int argc, char **argv, int(*AppMain)(int argc, char **argv), int *result)
 {
 	TTYBackend vtb(full_exe_path, std_in, std_out, ext_clipboard, norgb, nodetect, far2l_tty, esc_expiration, notify_pipe, result);
 
