@@ -5,15 +5,19 @@
 #define TIMER_ID     10
 
 // interval of timer that used to blink cursor and do some other things
-#define TIMER_PERIOD 500             // 0.5 second
+#define DEF_TIMER_PERIOD 500  // 0.1 second
+#define MIN_TIMER_PERIOD 100  // 0.1 second
+#define MAX_TIMER_PERIOD 500  // 0.5 second
 
 // time interval that used for deferred extra refresh after last title update
 // see comment on WinPortPanel::OnTitleChangedSync
-#define TIMER_EXTRA_REFRESH 100      // 0.1 second
+#define TIMER_EXTRA_REFRESH 100        // 0.1 second
 
 // how many timer ticks may pass since last input activity
 // before timer will be stopped until restarted by some activity
-#define TIMER_IDLING_CYCLES 60       // 0.5 second * 60 = 30 seconds
+//#define TIMER_IDLING_CYCLES 600       // 0.5 second * 60 = 30 seconds
+#define TIMER_IDLING_TIME (60000 * 3)  // 3 minutes
+
 
 // If time between adhoc text copy and mouse button release less then this value then text will not be copied. Used to protect against unwanted copy-paste-s
 #define QEDIT_COPY_MINIMAL_DELAY 150
@@ -23,14 +27,20 @@
 	#define WX_ALT_NONLATIN
 #endif
 
+//30000
+
 IConsoleOutput *g_winport_con_out = nullptr;
 IConsoleInput *g_winport_con_in = nullptr;
 bool g_broadway = false, g_wayland = false, g_remote = false;
+
 static int g_exit_code = 0;
 static int g_maximize = 0;
 static WinPortAppThread *g_winport_app_thread = NULL;
 static WinPortFrame *g_winport_frame = nullptr;
 
+static DWORD g_TIMER_PERIOD = DEF_TIMER_PERIOD;
+static DWORD g_TIMER_IDLING_CYCLES = TIMER_IDLING_TIME / DEF_TIMER_PERIOD;
+
 bool WinPortClipboard_IsBusy();
 
 static void NormalizeArea(SMALL_RECT &area)
@@ -80,6 +90,11 @@ static void DetectHostAbilities()
 		g_wayland = true;
 	}
 
+	const char *wayland_display = getenv("WAYLAND_DISPLAY");
+	if (wayland_display) {
+		g_wayland = true;
+	}
+
 	const char *ssh_conn = getenv("SSH_CONNECTION");
 	if (ssh_conn && *ssh_conn
 		&& strstr(ssh_conn, "127.0.0.") == NULL
@@ -108,6 +123,7 @@ extern "C" __attribute__ ((visibility("default"))) bool WinPortMainBackend(WinPo
 	MacInit();
 #endif
 
+
 	g_wx_norgb = a->norgb;
 	g_winport_con_out = a->winport_con_out;
 	g_winport_con_in = a->winport_con_in;
@@ -262,6 +278,7 @@ wxDEFINE_EVENT(WX_CONSOLE_ADHOC_QEDIT, wxCommandEvent);
 wxDEFINE_EVENT(WX_CONSOLE_SET_TWEAKS, wxCommandEvent);
 wxDEFINE_EVENT(WX_CONSOLE_CHANGE_FONT, wxCommandEvent);
 wxDEFINE_EVENT(WX_CONSOLE_SAVE_WIN_STATE, wxCommandEvent);
+wxDEFINE_EVENT(WX_CONSOLE_SET_CURSOR_BLINK_TIME, wxCommandEvent);
 wxDEFINE_EVENT(WX_CONSOLE_EXIT, wxCommandEvent);
 
 
@@ -530,6 +547,7 @@ wxBEGIN_EVENT_TABLE(WinPortPanel, wxPanel)
 	EVT_COMMAND(wxID_ANY, WX_CONSOLE_ADHOC_QEDIT, WinPortPanel::OnConsoleAdhocQuickEditSync)
 	EVT_COMMAND(wxID_ANY, WX_CONSOLE_SET_TWEAKS, WinPortPanel::OnConsoleSetTweaksSync)
 	EVT_COMMAND(wxID_ANY, WX_CONSOLE_CHANGE_FONT, WinPortPanel::OnConsoleChangeFontSync)
+	EVT_COMMAND(wxID_ANY, WX_CONSOLE_SET_CURSOR_BLINK_TIME, WinPortPanel::OnConsoleSetCursorBlinkTimeSync)
 	EVT_COMMAND(wxID_ANY, WX_CONSOLE_EXIT, WinPortPanel::OnConsoleExitSync)
 
 	EVT_IDLE(WinPortPanel::OnIdle)
@@ -556,7 +574,7 @@ WinPortPanel::WinPortPanel(WinPortFrame *frame, const wxPoint& pos, const wxSize
 	Create(frame, wxID_ANY, pos, size, wxWANTS_CHARS | wxNO_BORDER);
 	g_winport_con_out->SetBackend(this);
 	_periodic_timer = new wxTimer(this, TIMER_ID);
-	_periodic_timer->Start(TIMER_PERIOD);
+	_periodic_timer->Start(g_TIMER_PERIOD);
 	OnConsoleOutputTitleChanged();
 	_resize_pending = RP_INSTANT;
 }
@@ -734,7 +752,7 @@ void WinPortPanel::OnTimerPeriodic(wxTimerEvent& event)
 			_periodic_timer->Stop();
 			_extra_refresh = false;
 			Refresh();
-			_periodic_timer->Start(TIMER_PERIOD);
+			_periodic_timer->Start(g_TIMER_PERIOD);
 			fprintf(stderr, "Extra refresh\n");
 		}
 		return;
@@ -748,15 +766,15 @@ void WinPortPanel::OnTimerPeriodic(wxTimerEvent& event)
 	_paint_context.BlinkCursor();
 	++_timer_idling_counter;
 	// stop timer if counter reached limit and cursor is visible and no other timer-dependent things remained
-	if (_timer_idling_counter >= TIMER_IDLING_CYCLES && _paint_context.CursorBlinkState() && _text2clip.empty()) {
+	if (_timer_idling_counter >= g_TIMER_IDLING_CYCLES && _paint_context.CursorBlinkState() && _text2clip.empty()) {
 		_periodic_timer->Stop();
 	}
 }
 
 void WinPortPanel::ResetTimerIdling()
 {
-	if (_timer_idling_counter >= TIMER_IDLING_CYCLES && !_periodic_timer->IsRunning()) {
-		_periodic_timer->Start(_extra_refresh ? TIMER_EXTRA_REFRESH : TIMER_PERIOD);
+	if (_timer_idling_counter >= g_TIMER_IDLING_CYCLES && !_periodic_timer->IsRunning()) {
+		_periodic_timer->Start(_extra_refresh ? TIMER_EXTRA_REFRESH : g_TIMER_PERIOD);
 
 	} else if (_extra_refresh) {
 		_periodic_timer->Stop();
@@ -1597,7 +1615,7 @@ void WinPortPanel::OnConsoleSetTweaksSync( wxCommandEvent& event )
 
 DWORD64 WinPortPanel::OnConsoleSetTweaks(DWORD64 tweaks)
 {
-	DWORD64 out = TWEAK_STATUS_SUPPORT_CHANGE_FONT;
+	DWORD64 out = TWEAK_STATUS_SUPPORT_CHANGE_FONT | TWEAK_STATUS_SUPPORT_BLINK_RATE;
 
 	if (_paint_context.IsSharpSupported())
 		out|= TWEAK_STATUS_SUPPORT_PAINT_SHARP;
@@ -1716,6 +1734,30 @@ void WinPortPanel::OnConsoleExit()
 		wxQueueEvent(this, event);
 }
 
+void WinPortPanel::OnConsoleSetCursorBlinkTimeSync( wxCommandEvent& event )
+{
+	EventWithDWORD64 *e = (EventWithDWORD64 *)&event;
+	DWORD interval = e->cookie;
+	if (interval < 100 )
+		g_TIMER_PERIOD = 100;
+	else if (interval > 500 )
+		g_TIMER_PERIOD = 500;
+	else
+		g_TIMER_PERIOD = interval;
+
+	g_TIMER_IDLING_CYCLES = TIMER_IDLING_TIME / g_TIMER_PERIOD;
+
+	_periodic_timer->Stop();
+	_periodic_timer->Start(g_TIMER_PERIOD);
+}
+
+void WinPortPanel::OnConsoleSetCursorBlinkTime(DWORD interval)
+{
+	EventWithDWORD64 *event = new(std::nothrow) EventWithDWORD64(interval, WX_CONSOLE_SET_CURSOR_BLINK_TIME);
+	if (event)
+		wxQueueEvent(this, event);
+}
+
 void WinPortPanel::CheckPutText2CLip()
 {
 	if (!_text2clip.empty()) {
