@@ -67,6 +67,8 @@ THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "AllXLats.hpp"
 #include "ConfigOpt.hpp"
 #include "ConfigOptSaveLoad.hpp"
+#include "pick_color256.hpp"
+#include "pick_colorRGB.hpp"
 
 void SanitizeHistoryCounts();
 
@@ -120,6 +122,8 @@ static FARString strKeyNameConsoleDetachKey;
 
 const ConfigOpt g_cfg_opts[] {
 	{true,  NSecColors, "CurrentPalette", SIZE_ARRAY_PALETTE, Palette, DefaultPalette},
+	{true,  NSecColors, "TempColors256", TEMP_COLORS256_SIZE, g_tempcolors256, g_tempcolors256},
+	{true,  NSecColors, "TempColorsRGB", TEMP_COLORSRGB_SIZE, (BYTE *)g_tempcolorsRGB, (BYTE *)g_tempcolorsRGB},
 
 	{true,  NSecScreen, "Clock", &Opt.Clock, 1},
 	{true,  NSecScreen, "ViewerEditorClock", &Opt.ViewerEditorClock, 0},
@@ -127,6 +131,7 @@ const ConfigOpt g_cfg_opts[] {
 	{true,  NSecScreen, "ScreenSaver", &Opt.ScreenSaver, 0},
 	{true,  NSecScreen, "ScreenSaverTime", &Opt.ScreenSaverTime, 5},
 	{false, NSecScreen, "DeltaXY", &Opt.ScrSize.dwDeltaXY, 0},
+	{true,  NSecScreen, "CursorBlinkInterval", &Opt.CursorBlinkTime, 500},
 
 	{true,  NSecCmdline, "UsePromptFormat", &Opt.CmdLine.UsePromptFormat, 0},
 	{true,  NSecCmdline, "PromptFormat", &Opt.CmdLine.strPromptFormat, L"$p$# "},
@@ -379,7 +384,7 @@ const ConfigOpt g_cfg_opts[] {
 	{true,  NSecPanelLeft, "SortOrder", &Opt.LeftPanel.SortOrder, 1},
 	{true,  NSecPanelLeft, "SortGroups", &Opt.LeftPanel.SortGroups, 0},
 	{true,  NSecPanelLeft, "NumericSort", &Opt.LeftPanel.NumericSort, 0},
-	{true,  NSecPanelLeft, "CaseSensitiveSortNix", &Opt.LeftPanel.CaseSensitiveSort, 1},
+	{true,  NSecPanelLeft, "CaseSensitiveSort", &Opt.LeftPanel.CaseSensitiveSort, 0},
 	{true,  NSecPanelLeft, "Folder", &Opt.strLeftFolder, L""},
 	{true,  NSecPanelLeft, "CurFile", &Opt.strLeftCurFile, L""},
 	{true,  NSecPanelLeft, "SelectedFirst", &Opt.LeftSelectedFirst, 0},
@@ -393,7 +398,7 @@ const ConfigOpt g_cfg_opts[] {
 	{true,  NSecPanelRight, "SortOrder", &Opt.RightPanel.SortOrder, 1},
 	{true,  NSecPanelRight, "SortGroups", &Opt.RightPanel.SortGroups, 0},
 	{true,  NSecPanelRight, "NumericSort", &Opt.RightPanel.NumericSort, 0},
-	{true,  NSecPanelRight, "CaseSensitiveSortNix", &Opt.RightPanel.CaseSensitiveSort, 1},
+	{true,  NSecPanelRight, "CaseSensitiveSort", &Opt.RightPanel.CaseSensitiveSort, 0},
 	{true,  NSecPanelRight, "Folder", &Opt.strRightFolder, L""},
 	{true,  NSecPanelRight, "CurFile", &Opt.strRightCurFile, L""},
 	{true,  NSecPanelRight, "SelectedFirst", &Opt.RightSelectedFirst, 0},
@@ -580,6 +585,8 @@ void ConfigOptFromCmdLine()
 			FARString strName(pName, pVal - pName);
 			pVal++;
 			int index = ConfigOptGetIndex(strName.CPtr());
+			if (index<0)
+				continue;
 			switch (g_cfg_opts[index].type)
 			{
 				case ConfigOpt::T_DWORD:
@@ -641,6 +648,12 @@ void ConfigOptLoad()
 
 	SanitizeHistoryCounts();
 
+	if (Opt.CursorBlinkTime < 100)
+		Opt.CursorBlinkTime = 100;
+
+	if (Opt.CursorBlinkTime > 500)
+		Opt.CursorBlinkTime = 500;
+
 	if (Opt.ShowMenuBar)
 		Opt.ShowMenuBar = 1;
 
