@@ -310,6 +310,12 @@ void InterfaceSettings()
 		Builder.AddCheckbox(Msg::ConfigPgUpChangeDisk, &Opt.PgUpChangeDisk);
 
 		const DWORD supported_tweaks = ApplyConsoleTweaks();
+		if (supported_tweaks & TWEAK_STATUS_SUPPORT_BLINK_RATE) {
+
+			DialogItemEx *CursorEdit = Builder.AddIntEditField(&Opt.CursorBlinkTime, 3);
+			Builder.AddTextAfter(CursorEdit, Msg::ConfigCursorBlinkInt);
+		}
+
 		int ChangeFontID = -1;
 		DialogItemEx *ChangeFontItem = nullptr;
 		if (supported_tweaks & TWEAK_STATUS_SUPPORT_PAINT_SHARP) {
@@ -362,6 +368,8 @@ void InterfaceSettings()
 
 		WINPORT(ConsoleChangeFont)();
 	}
+
+	WINPORT(SetConsoleCursorBlinkTime)(NULL, Opt.CursorBlinkTime);
 }
 
 void AutoCompleteSettings()
