@@ -604,6 +604,8 @@ int FarAppMain(int argc, char **argv)
 
 	ConfigOptLoad();
 	InitConsole();
+	WINPORT(SetConsoleCursorBlinkTime)(NULL, Opt.CursorBlinkTime);
+
 	static_assert(!IsPtr(Msg::NewFileName._id),
 			"Too many language messages. Need to refactor code to eliminate use of IsPtr.");
 
@@ -760,6 +762,11 @@ int _cdecl main(int argc, char *argv[])
 
 	setlocale(LC_ALL, "");	// otherwise non-latin keys missing with XIM input method
 
+	const char *lcc = getenv("LC_COLLATE");
+	if (lcc && *lcc) {
+		setlocale(LC_COLLATE, lcc);
+	}
+
 	SetupFarPath(argv[0]);
 
 	{	// if CONFIG_INI is not present => first start & opt for show Help "FAR2L features - Getting Started"
