@@ -35,6 +35,8 @@ THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include <wchar.h>
 
 #include "setcolor.hpp"
+#include "pick_color256.hpp"
+#include "pick_colorRGB.hpp"
 #include "keys.hpp"
 #include "lang.hpp"
 #include "colors.hpp"
@@ -50,6 +52,8 @@ THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 #include "palette.hpp"
 #include "config.hpp"
 
+#include "pick_color.hpp"
+
 static void SetItemColors(MenuDataEx *Items, int *PaletteItems, int Size, int TypeSub);
 void GetColor(int PaletteIndex);
 static VMenu *MenuToRedraw1 = nullptr, *MenuToRedraw2 = nullptr, *MenuToRedraw3 = nullptr;
@@ -428,10 +432,10 @@ static void SetItemColors(MenuDataEx *Items, int *PaletteItems, int Size, int Ty
 void GetColor(int PaletteIndex)
 {
 	ChangeMacroMode chgMacroMode(MACRO_MENU);
-	WORD NewColor = Palette[PaletteIndex - COL_FIRSTPALETTECOLOR];
+	uint16_t NewColor = Palette[PaletteIndex - COL_FIRSTPALETTECOLOR];
 
-	if (GetColorDialog(NewColor)) {
-		Palette[PaletteIndex - COL_FIRSTPALETTECOLOR] = static_cast<BYTE>(NewColor);
+	if (GetColorDialog16(&NewColor, false)) {
+		Palette[PaletteIndex - COL_FIRSTPALETTECOLOR] = static_cast<uint16_t>(NewColor);
 		ScrBuf.Lock();	// отменяем всякую прорисовку
 		CtrlObject->Cp()->LeftPanel->Update(UPDATE_KEEP_SELECTION);
 		CtrlObject->Cp()->LeftPanel->Redraw();
@@ -462,308 +466,3 @@ void GetColor(int PaletteIndex)
 		FrameManager->PluginCommit();	// коммитим.
 	}
 }
-
-//////////////////////////////////////
-static wchar_t ColorDialogForeRGB[16], ColorDialogBackRGB[16];
-
-static void UpdateRGBFromDialog(HANDLE hDlg)
-{
-	SendDlgMessage(hDlg, DM_GETTEXTPTR, 36, (LONG_PTR)ColorDialogForeRGB);
-	SendDlgMessage(hDlg, DM_GETTEXTPTR, 38, (LONG_PTR)ColorDialogBackRGB);
-}
-
-static DWORD ReverseColorBytes(DWORD Color)
-{
-	return (Color & 0x00ff00) | ((Color >> 16) & 0xff) | ((Color & 0xff) << 16);
-}
-
-static DWORD ColorDialogForeRGBValue()
-{
-	return ReverseColorBytes(((DWORD)wcstoul(ColorDialogForeRGB, nullptr, 16)));
-}
-
-static DWORD ColorDialogBackRGBValue()
-{
-	return (ReverseColorBytes((DWORD)wcstoul(ColorDialogBackRGB, nullptr, 16)));
-}
-
-static DWORD64 ColorDialogForeRGBMask()
-{
-	return DWORD64(ColorDialogForeRGBValue()) << 16;
-}
-
-static DWORD64 ColorDialogBackRGBMask()
-{
-	return DWORD64(ColorDialogBackRGBValue()) << 40;
-}
-
-static void GetColorDlgProc_EnsureColorsAreInverted(SHORT x, SHORT y)
-{
-	CHAR_INFO ci{};
-	SMALL_RECT Rect = {x, y, x, y};
-	WINPORT(ReadConsoleOutput)(0, &ci, COORD{1, 1}, COORD{0, 0}, &Rect);
-	if (ci.Attributes & COMMON_LVB_REVERSE_VIDEO)
-		return;	// this cell is already tweaked during prev paint
-
-	DWORD64 InvColors = COMMON_LVB_REVERSE_VIDEO;
-
-	InvColors|= ((ci.Attributes & 0x0f) << 4) | ((ci.Attributes & 0xf0) >> 4);
-
-	InvColors|= (ci.Attributes & (COMMON_LVB_UNDERSCORE | COMMON_LVB_STRIKEOUT));
-
-	if (ci.Attributes & FOREGROUND_TRUECOLOR) {
-		SET_RGB_BACK(InvColors, GET_RGB_FORE(ci.Attributes));
-	}
-
-	if (ci.Attributes & BACKGROUND_TRUECOLOR) {
-		SET_RGB_FORE(InvColors, GET_RGB_BACK(ci.Attributes));
-	}
-
-	DWORD NumberOfAttrsWritten{};
-	WINPORT(FillConsoleOutputAttribute) (0, InvColors, 1, COORD{x, y}, &NumberOfAttrsWritten);
-}
-
-static void GetColorDlgProc_OnDrawn(HANDLE hDlg)
-{
-	UpdateRGBFromDialog(hDlg);
-
-	// Trick to fix #1392:
-	// For foreground-colored boxes invert Fg&Bg colors and add COMMON_LVB_REVERSE_VIDEO attribute
-	// this will put real colors on them if mapping of colors is different for Fg and Bg indexes
-
-	// Ensure everything is on screen and will use console API then
-	ScrBuf.Flush();
-
-	SMALL_RECT DlgRect{};
-	SendDlgMessage(hDlg, DM_GETDLGRECT, 0, (LONG_PTR)&DlgRect);
-	SMALL_RECT ItemRect{};
-
-	for (int ID = 2; ID <= 17; ++ID) {
-		if (SendDlgMessage(hDlg, DM_GETITEMPOSITION, ID, (LONG_PTR)&ItemRect)) {
-			ItemRect.Left+= DlgRect.Left;
-			ItemRect.Right+= DlgRect.Left;
-			ItemRect.Top+= DlgRect.Top;
-			ItemRect.Bottom+= DlgRect.Top;
-			for (auto x = ItemRect.Left; x <= ItemRect.Right; ++x) {
-				for (auto y = ItemRect.Top; y <= ItemRect.Bottom; ++y) {
-					GetColorDlgProc_EnsureColorsAreInverted(x, y);
-				}
-			}
-		}
-	}
-}
-
-static LONG_PTR GetColorDlgProc_CtlColorDlgItem(HANDLE hDlg, int ID, LONG_PTR DefaultColor, bool UseTrueColor)
-{
-	if (UseTrueColor) {
-		DialogItemTrueColors ditc{};
-		FarTrueColorFromRGB(ditc.Normal.Fore, ColorDialogForeRGBValue());
-		FarTrueColorFromRGB(ditc.Normal.Back, ColorDialogBackRGBValue());
-		SendDlgMessage(hDlg, DM_SETTRUECOLOR, ID, (LONG_PTR)&ditc);
-	}
-
-	int *CurColor = (int *)SendDlgMessage(hDlg, DM_GETDLGDATA, 0, 0);
-	return (DefaultColor & 0xFFFFFF00U) | ((*CurColor) & 0xFF);
-}
-
-static LONG_PTR WINAPI GetColorDlgProc(HANDLE hDlg, int Msg, int Param1, LONG_PTR Param2)
-{
-	switch (Msg) {
-		case DN_CTLCOLORDLGITEM:
-			if (Param1 >= 41 && Param1 <= 43) {
-				return GetColorDlgProc_CtlColorDlgItem(hDlg, Param1, Param2, Param1 >= 42);
-			}
-			break;
-
-		case DN_BTNCLICK:
-
-			if ( (Param1 >= 2 && Param1 <= 17) || (Param1 >= 19 && Param1 <= 34)) {
-				int NewColor;
-				int *CurColor = (int *)SendDlgMessage(hDlg, DM_GETDLGDATA, 0, 0);
-				FarDialogItem *DlgItem =
-						(FarDialogItem *)malloc(SendDlgMessage(hDlg, DM_GETDLGITEM, Param1, 0));
-				SendDlgMessage(hDlg, DM_GETDLGITEM, Param1, (LONG_PTR)DlgItem);
-				NewColor = *CurColor;
-
-				if (Param1 <= 17)		// Fore
-				{
-					NewColor&= ~0x0F;
-					NewColor|= (DlgItem->Flags & B_MASK) >> 4;
-				} else // if (Param1 >= 19) // Back 
-				{
-					NewColor&= ~0xF0;
-					NewColor|= DlgItem->Flags & B_MASK;
-				}
-
-				if (NewColor != *CurColor)
-					*CurColor = NewColor;
-
-				free(DlgItem);
-				return TRUE;
-			}
-			break;
-
-		case DN_DRAWDIALOGDONE:
-			GetColorDlgProc_OnDrawn(hDlg);
-			break;
-
-		case DN_EDITCHANGE:
-			if (Param1 == 36 || Param1 == 38) {
-				GetColorDlgProc_OnDrawn(hDlg);
-			}
-			break;
-
-		case DN_CLOSE:
-			UpdateRGBFromDialog(hDlg);
-			break;
-	}
-
-	return DefDlgProc(hDlg, Msg, Param1, Param2);
-}
-
-static bool GetColorDialogInner(bool bForFileFilter, DWORD64 &Color, bool bCentered)
-{
-	const wchar_t *HexMask = L"HHHHHH";
-	swprintf(ColorDialogForeRGB, ARRAYSIZE(ColorDialogForeRGB), L"%06X", ReverseColorBytes((Color >> 16) & 0xffffff));
-	swprintf(ColorDialogBackRGB, ARRAYSIZE(ColorDialogBackRGB), L"%06X", ReverseColorBytes((Color >> 40) & 0xffffff));
-
-	DialogDataEx ColorDlgData[] = {
-		/*   0 */ {DI_DOUBLEBOX, 3, 1, 39, 14, {}, 0, Msg::SetColorTitle},
-
-		/*   1 */ {DI_SINGLEBOX, 5, 2, 20, 7, {}, 0, Msg::SetColorForeground},
-		/*   2 */ {DI_RADIOBUTTON, 7, 3, 0, 3, {}, F_LIGHTGRAY | B_BLACK | DIF_GROUP | DIF_SETCOLOR | DIF_MOVESELECT, L""},
-		/*   3 */ {DI_RADIOBUTTON, 7, 4, 0, 4, {}, F_BLACK | B_RED | DIF_SETCOLOR | DIF_MOVESELECT, L""},
-		/*   4 */ {DI_RADIOBUTTON, 7, 5, 0, 5, {}, F_LIGHTGRAY | B_DARKGRAY | DIF_SETCOLOR | DIF_MOVESELECT, L""},
-		/*   5 */ {DI_RADIOBUTTON, 7, 6, 0, 6, {}, F_BLACK | B_LIGHTRED | DIF_SETCOLOR | DIF_MOVESELECT, L""},
-		/*   6 */ {DI_RADIOBUTTON, 10, 3, 0, 3, {}, F_LIGHTGRAY | B_BLUE | DIF_SETCOLOR | DIF_MOVESELECT, L""},
-		/*   7 */ {DI_RADIOBUTTON, 10, 4, 0, 4, {}, F_BLACK | B_MAGENTA | DIF_SETCOLOR | DIF_MOVESELECT, L""},
-		/*   8 */ {DI_RADIOBUTTON, 10, 5, 0, 5, {}, F_BLACK | B_LIGHTBLUE | DIF_SETCOLOR | DIF_MOVESELECT, L""},
-		/*   9 */ {DI_RADIOBUTTON, 10, 6, 0, 6, {}, F_BLACK | B_LIGHTMAGENTA | DIF_SETCOLOR | DIF_MOVESELECT, L""},
-		/*  10 */ {DI_RADIOBUTTON, 13, 3, 0, 3, {}, F_BLACK | B_GREEN | DIF_SETCOLOR | DIF_MOVESELECT, L""},
-		/*  11 */ {DI_RADIOBUTTON, 13, 4, 0, 4, {}, F_BLACK | B_BROWN | DIF_SETCOLOR | DIF_MOVESELECT, L""},
-		/*  12 */ {DI_RADIOBUTTON, 13, 5, 0, 5, {}, F_BLACK | B_LIGHTGREEN | DIF_SETCOLOR | DIF_MOVESELECT, L""},
-		/*  13 */ {DI_RADIOBUTTON, 13, 6, 0, 6, {}, F_BLACK | B_YELLOW | DIF_SETCOLOR | DIF_MOVESELECT, L""},
-		/*  14 */ {DI_RADIOBUTTON, 16, 3, 0, 3, {}, F_BLACK | B_CYAN | DIF_SETCOLOR | DIF_MOVESELECT, L""},
-		/*  15 */ {DI_RADIOBUTTON, 16, 4, 0, 4, {}, F_BLACK | B_LIGHTGRAY | DIF_SETCOLOR | DIF_MOVESELECT, L""},
-		/*  16 */ {DI_RADIOBUTTON, 16, 5, 0, 5, {}, F_BLACK | B_LIGHTCYAN | DIF_SETCOLOR | DIF_MOVESELECT, L""},
-		/*  17 */ {DI_RADIOBUTTON, 16, 6, 0, 6, {}, F_BLACK | B_WHITE | DIF_SETCOLOR | DIF_MOVESELECT, L""},
-
-		/*  18 */ {DI_SINGLEBOX, 22, 2, 37, 7, {}, 0, Msg::SetColorBackground},
-		/*  19 */ {DI_RADIOBUTTON, 24, 3, 0, 3, {}, F_LIGHTGRAY | B_BLACK | DIF_GROUP | DIF_SETCOLOR | DIF_MOVESELECT, L""},
-		/*  20 */ {DI_RADIOBUTTON, 24, 4, 0, 4, {}, F_BLACK | B_RED | DIF_SETCOLOR | DIF_MOVESELECT, L""},
-		/*  21 */ {DI_RADIOBUTTON, 24, 5, 0, 5, {}, F_LIGHTGRAY | B_DARKGRAY | DIF_SETCOLOR | DIF_MOVESELECT, L""},
-		/*  22 */ {DI_RADIOBUTTON, 24, 6, 0, 6, {}, F_BLACK | B_LIGHTRED | DIF_SETCOLOR | DIF_MOVESELECT, L""},
-		/*  23 */ {DI_RADIOBUTTON, 27, 3, 0, 3, {}, F_LIGHTGRAY | B_BLUE | DIF_SETCOLOR | DIF_MOVESELECT, L""},
-		/*  24 */ {DI_RADIOBUTTON, 27, 4, 0, 4, {}, F_BLACK | B_MAGENTA | DIF_SETCOLOR | DIF_MOVESELECT, L""},
-		/*  25 */ {DI_RADIOBUTTON, 27, 5, 0, 5, {}, F_BLACK | B_LIGHTBLUE | DIF_SETCOLOR | DIF_MOVESELECT, L""},
-		/*  26 */ {DI_RADIOBUTTON, 27, 6, 0, 6, {}, F_BLACK | B_LIGHTMAGENTA | DIF_SETCOLOR | DIF_MOVESELECT, L""},
-		/*  27 */ {DI_RADIOBUTTON, 30, 3, 0, 3, {}, F_BLACK | B_GREEN | DIF_SETCOLOR | DIF_MOVESELECT, L""},
-		/*  28 */ {DI_RADIOBUTTON, 30, 4, 0, 4, {}, F_BLACK | B_BROWN | DIF_SETCOLOR | DIF_MOVESELECT, L""},
-		/*  29 */ {DI_RADIOBUTTON, 30, 5, 0, 5, {}, F_BLACK | B_LIGHTGREEN | DIF_SETCOLOR | DIF_MOVESELECT, L""},
-		/*  30 */ {DI_RADIOBUTTON, 30, 6, 0, 6, {}, F_BLACK | B_YELLOW | DIF_SETCOLOR | DIF_MOVESELECT, L""},
-		/*  31 */ {DI_RADIOBUTTON, 33, 3, 0, 3, {}, F_BLACK | B_CYAN | DIF_SETCOLOR | DIF_MOVESELECT, L""},
-		/*  32 */ {DI_RADIOBUTTON, 33, 4, 0, 4, {}, F_BLACK | B_LIGHTGRAY | DIF_SETCOLOR | DIF_MOVESELECT, L""},
-		/*  33 */ {DI_RADIOBUTTON, 33, 5, 0, 5, {}, F_BLACK | B_LIGHTCYAN | DIF_SETCOLOR | DIF_MOVESELECT, L""},
-		/*  34 */ {DI_RADIOBUTTON, 33, 6, 0, 6, {}, F_BLACK | B_WHITE | DIF_SETCOLOR | DIF_MOVESELECT, L""},
-
-		/*  35 */ {DI_TEXT,         7,   7,  11, 7,  {}, DIF_HIDDEN, L"RGB#:"},
-		/*  36 */ {DI_FIXEDIT,     13,   7,  18, 7,  {(DWORD_PTR)HexMask}, DIF_HIDDEN | DIF_MASKEDIT, ColorDialogForeRGB},
-		/*  37 */ {DI_TEXT,        24,   7,  28, 7,  {}, DIF_HIDDEN, L"RGB#:"},
-		/*  38 */ {DI_FIXEDIT,     30,   7,  35, 7,  {(DWORD_PTR)HexMask}, DIF_HIDDEN | DIF_MASKEDIT, ColorDialogBackRGB},
-
-		/*  39 */ {DI_CHECKBOX,  5, 9, 0, 9, {}, DIF_HIDDEN, Msg::SetColorForeTransparent},
-		/*  40 */ {DI_CHECKBOX, 22, 9, 0, 9, {}, DIF_HIDDEN, Msg::SetColorBackTransparent},
-
-		/*  41 */ {DI_TEXT,   5,   9, 37,  9, {}, DIF_SETCOLOR, Msg::SetColorSample},
-		/*  42 */ {DI_TEXT,   5,  10, 37, 10, {}, DIF_SETCOLOR, Msg::SetColorSample},
-		/*  43 */ {DI_TEXT,   5,  11, 37, 11, {}, DIF_SETCOLOR, Msg::SetColorSample},
-		/*  44 */ {DI_TEXT,   0,  12,  0, 12, {}, DIF_SEPARATOR, L""},
-		/*  45 */ {DI_BUTTON, 0,  13,  0, 13, {}, DIF_DEFAULT | DIF_CENTERGROUP, Msg::SetColorSet},
-		/*  46 */ {DI_BUTTON, 0,  13,  0, 13, {}, DIF_CENTERGROUP, Msg::SetColorCancel},
-	};
-	MakeDialogItemsEx(ColorDlgData, ColorDlg);
-	int ExitCode;
-	WORD CurColor = Color;
-
-	for (size_t i = 2; i <= 17; i++) {
-		if (static_cast<WORD>((ColorDlg[i].Flags & B_MASK) >> 4) == (Color & F_MASK)) {
-			ColorDlg[i].Selected = 1;
-			ColorDlg[i].Focus = TRUE;
-			break;
-		}
-	}
-
-	for (size_t i = 19; i <= 34; i++) {
-		if (static_cast<WORD>(ColorDlg[i].Flags & B_MASK) == (Color & B_MASK)) {
-			ColorDlg[i].Selected = 1;
-			break;
-		}
-	}
-
-	for (size_t i = 41; i <= 43; i++) {
-		ColorDlg[i].Flags = (ColorDlg[i].Flags & ~DIF_COLORMASK) | (Color & 0xffff);
-	}
-
-	if (bForFileFilter) {
-		ColorDlg[0].Y2+= 2;
-		ColorDlg[1].Y2++;
-		ColorDlg[18].Y2++;
-
-		for (size_t i = 35; i <= 40; ++i) {
-			ColorDlg[i].Flags&= ~DIF_HIDDEN;
-		}
-		for (size_t i = 41; i <= 46; i++) {
-			ColorDlg[i].Y1+= 2;
-			ColorDlg[i].Y2+= 2;
-		}
-		ColorDlg[39].Selected = (Color & 0x0F00 ? 1 : 0);
-		ColorDlg[40].Selected = (Color & 0xF000 ? 1 : 0);
-	}
-
-	{
-		Dialog Dlg(ColorDlg, ARRAYSIZE(ColorDlg), GetColorDlgProc, (LONG_PTR)&CurColor);
-
-		if (bCentered)
-			Dlg.SetPosition(-1, -1, 39 + 4, 15 + (bForFileFilter ? 3 : 2));
-		else
-			Dlg.SetPosition(37, 2, 75 + 4, 16 + (bForFileFilter ? 3 : 2));
-
-		Dlg.Process();
-		ExitCode = Dlg.GetExitCode();
-	}
-
-	if (ExitCode == 45) {
-		Color = CurColor & 0xffff;
-
-		if (ColorDlg[39].Selected)
-			Color|= 0x0F00;
-		else
-			Color&= 0xF0FF;
-
-		if (ColorDlg[40].Selected)
-			Color|= 0xF000;
-		else
-			Color&= 0x0FFF;
-
-		Color|= ColorDialogForeRGBMask();
-		Color|= ColorDialogBackRGBMask();
-
-		return true;
-	}
-
-	return false;
-}
-
-bool GetColorDialogForFileFilter(DWORD64 &Color)
-{
-	return GetColorDialogInner(true, Color, true);
-}
-
-bool GetColorDialog(WORD &Color, bool bCentered)
-{
-	DWORD64 ColorRGB = Color;
-	bool out = GetColorDialogInner(false, ColorRGB, bCentered);
-	Color = ColorRGB & 0xffff;
-	return out;
-}
-
